@@ -1,15 +1,19 @@
-# BSD 3-Clause License; see https://github.com/scikit-hep/awkward-1.0/blob/main/LICENSE
-__all__ = ("argcartesian",)
+# BSD 3-Clause License; see https://github.com/scikit-hep/awkward/blob/main/LICENSE
+
+from __future__ import annotations
+
+from collections.abc import Mapping
+
 import awkward as ak
-from awkward._behavior import behavior_of
-from awkward._layout import wrap_layout
-from awkward._nplikes.numpylike import NumpyMetadata
-from awkward._regularize import regularize_axis
+from awkward._dispatch import high_level_function
+from awkward._nplikes.numpy_like import NumpyMetadata
+
+__all__ = ("argcartesian",)
 
 np = NumpyMetadata.instance()
-cpu = ak._backends.NumpyBackend.instance()
 
 
+@high_level_function()
 def argcartesian(
     arrays,
     axis=1,
@@ -19,11 +23,12 @@ def argcartesian(
     with_name=None,
     highlevel=True,
     behavior=None,
+    attrs=None,
 ):
     """
     Args:
-        arrays (dict or iterable of arrays): Each value in this dict or iterable
-            can be any array-like data that #ak.to_layout recognizes.
+        arrays (mapping or sequence of arrays): Each value in this mapping or
+            sequence can be any array-like data that #ak.to_layout recognizes.
         axis (int): The dimension at which this operation is applied. The
             outermost dimension is `0`, followed by `1`, etc., and negative
             values count backward from the innermost: `-1` is the innermost
@@ -44,6 +49,8 @@ def argcartesian(
             a low-level #ak.contents.Content subclass.
         behavior (None or dict): Custom #ak.behavior for the output array, if
             high-level.
+        attrs (None or dict): Custom attributes for the output array, if
+            high-level.
 
     Computes a Cartesian product (i.e. cross product) of data from a set of
     `arrays`, like #ak.cartesian, but returning integer indexes for
@@ -86,45 +93,23 @@ def argcartesian(
     All of the parameters for #ak.cartesian apply equally to #ak.argcartesian,
     so see the #ak.cartesian documentation for a more complete description.
     """
-    with ak._errors.OperationErrorContext(
-        "ak.argcartesian",
-        {
-            "arrays": arrays,
-            "axis": axis,
-            "nested": nested,
-            "parameters": parameters,
-            "with_name": with_name,
-            "highlevel": highlevel,
-            "behavior": behavior,
-        },
-    ):
-        return _impl(arrays, axis, nested, parameters, with_name, highlevel, behavior)
-
-
-def _impl(arrays, axis, nested, parameters, with_name, highlevel, behavior):
-    axis = regularize_axis(axis)
-
-    if isinstance(arrays, dict):
-        backend = ak._backends.backend_of(*arrays.values(), default=cpu)
-        behavior = behavior_of(*arrays.values(), behavior=behavior)
-        layouts = {
-            n: ak._do.local_index(
-                ak.operations.to_layout(x, allow_record=False, allow_other=False),
-                axis,
-            ).to_backend(backend)
-            for n, x in arrays.items()
-        }
+    # Dispatch
+    if isinstance(arrays, Mapping):
+        yield arrays.values()
     else:
-        arrays = list(arrays)
-        backend = ak._backends.backend_of(*arrays, default=cpu)
-        behavior = behavior_of(*arrays, behavior=behavior)
-        layouts = [
-            ak._do.local_index(
-                ak.operations.to_layout(x, allow_record=False, allow_other=False),
-                axis,
-            ).to_backend(backend)
-            for x in arrays
-        ]
+        yield arrays
+
+    # Implementation
+    return _impl(
+        arrays, axis, nested, parameters, with_name, highlevel, behavior, attrs
+    )
+
+
+def _impl(arrays, axis, nested, parameters, with_name, highlevel, behavior, attrs):
+    if isinstance(arrays, Mapping):
+        index_arrays = {n: ak.local_index(x, axis) for n, x in arrays.items()}
+    else:
+        index_arrays = [ak.local_index(x) for x in arrays]
 
     if with_name is not None:
         if parameters is None:
@@ -133,13 +118,11 @@ def _impl(arrays, axis, nested, parameters, with_name, highlevel, behavior):
             parameters = dict(parameters)
         parameters["__record__"] = with_name
 
-    result = ak.operations.cartesian(
-        layouts,
+    return ak.operations.cartesian(
+        index_arrays,
         axis=axis,
         nested=nested,
         parameters=parameters,
-        highlevel=False,
+        highlevel=highlevel,
         behavior=behavior,
     )
-
-    return wrap_layout(result, behavior, highlevel)
