@@ -1,14 +1,26 @@
-# BSD 3-Clause License; see https://github.com/scikit-hep/awkward-1.0/blob/main/LICENSE
-__all__ = ("flatten",)
+# BSD 3-Clause License; see https://github.com/scikit-hep/awkward/blob/main/LICENSE
+
+from __future__ import annotations
+
 import awkward as ak
-from awkward._layout import maybe_posaxis, wrap_layout
-from awkward._nplikes.numpylike import NumpyMetadata
+from awkward._dispatch import high_level_function
+from awkward._layout import HighLevelContext, maybe_posaxis
+from awkward._namedaxis import (
+    _get_named_axis,
+    _keep_named_axis,
+    _named_axis_to_positional_axis,
+    _remove_named_axis,
+)
+from awkward._nplikes.numpy_like import NumpyMetadata
 from awkward._regularize import regularize_axis
 
+__all__ = ("flatten",)
+
 np = NumpyMetadata.instance()
 
 
-def flatten(array, axis=1, *, highlevel=True, behavior=None):
+@high_level_function()
+def flatten(array, axis=1, *, highlevel=True, behavior=None, attrs=None):
     """
     Args:
         array: Array-like data (anything #ak.to_layout recognizes).
@@ -22,6 +34,8 @@ def flatten(array, axis=1, *, highlevel=True, behavior=None):
             a low-level #ak.contents.Content subclass.
         behavior (None or dict): Custom #ak.behavior for the output array, if
             high-level.
+        attrs (None or dict): Custom attributes for the output array, if
+            high-level.
 
     Returns an array with one level of nesting removed by erasing the
     boundaries between consecutive lists. Since this operates on a level of
@@ -157,16 +171,32 @@ def flatten(array, axis=1, *, highlevel=True, behavior=None):
          2.2,
          999]
     """
-    with ak._errors.OperationErrorContext(
-        "ak.flatten",
-        {"array": array, "axis": axis, "highlevel": highlevel, "behavior": behavior},
-    ):
-        return _impl(array, axis, highlevel, behavior)
+    # Dispatch
+    yield (array,)
+
+    # Implementation
+    return _impl(array, axis, highlevel, behavior, attrs)
 
 
-def _impl(array, axis, highlevel, behavior):
-    axis = regularize_axis(axis)
-    layout = ak.operations.to_layout(array, allow_record=False, allow_other=False)
+def _impl(array, axis, highlevel, behavior, attrs):
+    with HighLevelContext(behavior=behavior, attrs=attrs) as ctx:
+        layout = ctx.unwrap(array, allow_record=False, primitive_policy="error")
+
+    # Handle named axis
+    named_axis = _get_named_axis(ctx)
+    # Step 1: Normalize named axis to positional axis
+    axis = _named_axis_to_positional_axis(named_axis, axis)
+    axis = regularize_axis(axis, none_allowed=True)
+    # Step 2: propagate named axis from input to output,
+    #   if axis == None: use strategy "remove all" (see: awkward._namedaxis)
+    #   if axis == 0: use strategy "keep all" (see: awkward._namedaxis)
+    #   if axis != 0: use strategy "remove one" (see: awkward._namedaxis)
+    if axis is None:
+        pass
+    elif axis == 0 or maybe_posaxis(layout, axis, 1) == 0:
+        out_named_axis = _keep_named_axis(named_axis, None)
+    else:
+        out_named_axis = _remove_named_axis(named_axis, axis, layout.minmax_depth[1])
 
     if axis is None:
         out = ak._do.remove_structure(layout, function_name="ak.flatten")
@@ -174,17 +204,13 @@ def _impl(array, axis, highlevel, behavior):
             isinstance(x, ak.contents.Content) for x in out
         )
 
-        result = ak._do.mergemany(out)
-
-        return wrap_layout(result, behavior, highlevel)
+        out = ak._do.mergemany(out)
 
     elif axis == 0 or maybe_posaxis(layout, axis, 1) == 0:
 
         def apply(layout):
-            backend = layout.backend
-
             if layout.is_unknown:
-                return apply(ak.contents.NumpyArray(backend.nplike.asarray([])))
+                return apply(ak.contents.NumpyArray(layout.backend.nplike.asarray([])))
 
             elif layout.is_indexed:
                 return apply(layout.project())
@@ -196,20 +222,20 @@ def _impl(array, axis, highlevel, behavior):
                 ):
                     return layout
 
-                tags = backend.index_nplike.asarray(layout.tags)
-                index = backend.index_nplike.asarray(
-                    backend.nplike.asarray(layout.index), copy=True
+                tags = layout.tags.data
+                index = layout.backend.nplike.asarray(layout.index.data, copy=True)
+                big_mask = layout.backend.nplike.empty(
+                    layout.index.length, dtype=np.bool_
                 )
-                bigmask = backend.index_nplike.empty(len(index), dtype=np.bool_)
                 for tag, content in enumerate(layout.contents):
                     if content.is_option and not isinstance(
                         content, ak.contents.UnmaskedArray
                     ):
-                        bigmask[:] = False
-                        bigmask[tags == tag] = backend.index_nplike.asarray(
-                            content.mask_as_bool(valid_when=False)
+                        big_mask[:] = False
+                        big_mask[tags == tag] = content.mask_as_bool(
+                            valid_when=False
                         ).view(np.bool_)
-                        index[bigmask] = -1
+                        index[big_mask] = -1
 
                 good = index >= 0
                 return ak.contents.UnionArray(
@@ -225,9 +251,29 @@ def _impl(array, axis, highlevel, behavior):
                 return layout
 
         out = apply(layout)
-
-        return wrap_layout(out, behavior, highlevel, like=array)
-
     else:
         out = ak._do.flatten(layout, axis)
-        return wrap_layout(out, behavior, highlevel, like=array)
+
+    wrapped_out = ctx.wrap(
+        out,
+        highlevel=highlevel,
+    )
+
+    # propagate named axis to output
+    #   if axis == None: use strategy "remove all" (see: awkward._namedaxis)
+    if axis is None:
+        return ak.operations.ak_without_named_axis._impl(
+            wrapped_out,
+            highlevel=highlevel,
+            behavior=ctx.behavior,
+            attrs=ctx.attrs,
+        )
+    #   if axis == 0: use strategy "keep all" (see: awkward._namedaxis)
+    #   if axis != 0: use strategy "remove one" (see: awkward._namedaxis)
+    return ak.operations.ak_with_named_axis._impl(
+        wrapped_out,
+        named_axis=out_named_axis,
+        highlevel=highlevel,
+        behavior=ctx.behavior,
+        attrs=ctx.attrs,
+    )
